//+------------------------------------------------------------------+
//|                                    Guppy Mulitple Moving Average |
//+------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 6

//
//
//
//
//

extern string TimeFrame    = "current time frame";
extern bool   ShowLongGMMA = false;
extern bool   Interpolate  = true;
extern int    LinesWidth   = 1;
extern color  StartColor   = Lime;
extern color  EndColor     = Orange;

//
//
//
//
//

double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];
double Buffer5[];
double Buffer6[];

//
//
//
//
//

double periodsShort[]={ 3, 5, 8,10,12,15};
double periodsLong[] ={30,35,40,45,50,60};
double periods[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexBuffer(2,Buffer3);
   SetIndexBuffer(3,Buffer4);
   SetIndexBuffer(4,Buffer5);
   SetIndexBuffer(5,Buffer6);
   if (ShowLongGMMA)
         ArrayCopy(periods,periodsLong);
   else  ArrayCopy(periods,periodsShort);

   //
   //
   //
   //
   //

   indicatorFileName = WindowExpertName();
   returnBars        = (TimeFrame=="returnBars"); if (returnBars) return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
      IndicatorShortName("GMMA "+timeFrameToString(timeFrame));
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   static bool initialized=false;
   int i,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
      limit=MathMin(Bars-counted_bars,Bars-1);
      
      //
      //
      //
      //
      //
      
      if (returnBars)  { Buffer1[0] = limit+1; return(0); }
      if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      if (!initialized)
      {
         initialized=true;
            for (int k = 0; k<6; k++) SetIndexStyle(k,EMPTY,EMPTY,LinesWidth,gradientColor(k,6,StartColor,EndColor));
      }            
   
   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Buffer1[i]=iMA(NULL,timeFrame,periods[0],0,MODE_EMA,PRICE_CLOSE,y);
         Buffer2[i]=iMA(NULL,timeFrame,periods[1],0,MODE_EMA,PRICE_CLOSE,y);
         Buffer3[i]=iMA(NULL,timeFrame,periods[2],0,MODE_EMA,PRICE_CLOSE,y);
         Buffer4[i]=iMA(NULL,timeFrame,periods[3],0,MODE_EMA,PRICE_CLOSE,y);
         Buffer5[i]=iMA(NULL,timeFrame,periods[4],0,MODE_EMA,PRICE_CLOSE,y);
         Buffer6[i]=iMA(NULL,timeFrame,periods[5],0,MODE_EMA,PRICE_CLOSE,y);
         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(k = 1; k < n; k++)
            {
               Buffer1[i+k] = k*factor*Buffer1[i+n] + (1.0-k*factor)*Buffer1[i];
               Buffer2[i+k] = k*factor*Buffer2[i+n] + (1.0-k*factor)*Buffer2[i];
               Buffer3[i+k] = k*factor*Buffer3[i+n] + (1.0-k*factor)*Buffer3[i];
               Buffer4[i+k] = k*factor*Buffer4[i+n] + (1.0-k*factor)*Buffer4[i];
               Buffer5[i+k] = k*factor*Buffer5[i+n] + (1.0-k*factor)*Buffer5[i];
               Buffer6[i+k] = k*factor*Buffer6[i+n] + (1.0-k*factor)*Buffer6[i];
            }
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

color gradientColor(int step, int totalSteps, color from, color to)
{
   color newBlue  = getColor(step,totalSteps,(from & 0XFF0000)>>16,(to & 0XFF0000)>>16)<<16;
   color newGreen = getColor(step,totalSteps,(from & 0X00FF00)>> 8,(to & 0X00FF00)>> 8) <<8;
   color newRed   = getColor(step,totalSteps,(from & 0X0000FF)    ,(to & 0X0000FF)    )    ;
   return(newBlue+newGreen+newRed);
}
color getColor(int stepNo, int totalSteps, color from, color to)
{
   double step = (from-to)/(totalSteps-1.0);
   return(MathRound(from-step*stepNo));
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}